<# 
Credit: Script inspired from Manjunath 

PURPOSE OF THE SCRIPT: Synchronize user data from Azure Active Directory to a SharePoint list
ROOTFUSION ADDRESS BOOK WEB PART can be used to create the SharePoint list and nicely display your directory
THE SCRIPT CAN ALSO GENERATE AN EXCEL FILE

INPUT: encrypted credential password
OUTPUT: excel file
 
Pre-Requisites: 
This script needs 'SharePointPnPPowerShellOnline' and 'AzureAD' PowerShell modules 
 
#Install-Module SharePointPnPPowerShellOnline
#Install-Module AzureAD -Force
 
#> 


#Config Variables
#Set LimitUsers to -1 to get all users
$LimitUsers=-1
#Username running the script and allowed to query your Azure Active Directory
$Username = "scriptusername@contoso.com"
#Path to file containing the encrypted password of the user running the script.
#To create the encrypted password file, use the script CreateEncryptedPW.ps1
$SecurePwPath = "C:\scripts\AADS.txt"
#SharePoint site URL where stand your SharePoint list: uncomment and set your site URL
# $SiteURL = "https://contoso.sharepoint.com/intranetsite"
#SharePoint list name - use the ROOTFUSION Active Directory Address Book web part to create the list (with the management console)
$ListName ="ADList"
#Keep ExcelFileName empty to skip Excel file creation ($ExcelFileName ="")
$ExcelFileName ="AzureADUserList.xlsx"
$ExcelFolderPath = "C:\scripts"
$FilterOnlyAccountEnabled = $true
$FilterExcludeGuest = $true
$RemoveNotFoundADUsersFromSP = $true
$FilterPhoneRequired = $true

#Load Pw
$SecurePassword = Get-Content $SecurePwPath | ConvertTo-SecureString

#Build Credentials
$UserCredential = New-Object System.Management.Automation.PSCredential -ArgumentList $Username,$SecurePassword

#Connect to SharePoint site with PNP
Connect-PnPOnline -Url $SiteURL -Credentials $UserCredential

#Create Excel object if needed
if ($ExcelFileName -ne "")
{
  try
  { 
    # Create an Excel COM Object 
    $excel = New-Object -ComObject excel.application 
  }
  catch
  { 
    Write-Host "Something went wrong in creating excel. Make sure you have MSOffice installed to access MSExcel. Please try running the script again. `n" -ForegroundColor Yellow 
  } 

  # Create a Workbook 
  $workbook = $excel.Workbooks.Add() 
 
  # Creating a directory, overrides if any directory exists with the same name 
  Write-Host "Creating a directory. This operation will override if you have a directory with the same name. `n" -ForegroundColor Yellow 
  New-Item $ExcelFolderPath -Type Directory -Force 
}

#Connect to Azure
Connect-AzureAD -Credential $UserCredential

#Build filters
if ($FilterOnlyAccountEnabled) {
  $filterString="accountEnabled eq true"
} else {
  $filterString=""
}

if ($FilterExcludeGuest)
{
  if ($filerString -ne "") {
    $filterstring=$filterString + " and "
  }
  $filterString=$filterString + "UserType eq 'Member'"
}

#Query the AD
if ($LimitUsers -eq -1)
{
  $users = Get-AzureADUser -All 1 -filter $filterString  
} else {
  $users = Get-AzureADUser -Top $LimitUsers -filter $filterString
}

########################################################### 
#    Function to create Azure AD User List                # 
########################################################### 
 
function CreateAzureUserList { 
  $ExcelFileName = $args[0]
  $ExcelFolderPath = $args[1]
  $FilterPhoneRequired = $args[2]

  #Building up current list of users from SharePoint  
  $CurrentList = @() 
  $CurrentList=(Get-PnPListItem -List $ListName -Fields "Title","Display_Name","Surname","GivenName","CompanyName","JobTitle","Department","EMail","TelephoneNumber","Mobile","StreetAddress","City","State","PostalCode","Country","ObjectID","UserPrincipalName","Manager").FieldValues
  $FoundList = @()
  $i=0
  foreach ($Currenlist_user in $CurrentList)
  {
    $FoundList += 0
    $i++
  }

  Write-host "Found items in SP List: " $i 
 
  Write-Host "Creating the Azure Active Directory User List..." -ForegroundColor Green 
         
  # Adding worksheet 
  if ($ExcelFileName -ne "") 
  {
    $workbook.Worksheets.Add() 
    # Creating the "Azure AD User List" worksheet and naming it 
    $AzureADUserListWorksheet = $workbook.Worksheets.Item(1) 
    $AzureADUserListWorksheet.Name = 'Azure AD User List' 
 
    # Headers for the worksheet 
    $AzureADUserListWorksheet.Cells.Item(1,1) = 'ObjectID' 
    $AzureADUserListWorksheet.Cells.Item(1,2) = 'UserPrincipalName' 
    $AzureADUserListWorksheet.Cells.Item(1,3) = 'Display_Name' 
    $AzureADUserListWorksheet.Cells.Item(1,4) = 'GivenName' 
    $AzureADUserListWorksheet.Cells.Item(1,5) = 'Surname' 
    $AzureADUserListWorksheet.Cells.Item(1,6) = 'Department' 
    $AzureADUserListWorksheet.Cells.Item(1,7) = 'JobTitle'
    $AzureADUserListWorksheet.Cells.Item(1,8) = 'EMail' 
    $AzureADUserListWorksheet.Cells.Item(1,9) = 'StreetAddress' 
    $AzureADUserListWorksheet.Cells.Item(1,10) = 'City' 
    $AzureADUserListWorksheet.Cells.Item(1,11) = 'State' 
    $AzureADUserListWorksheet.Cells.Item(1,12) = 'PostalCode' 
    $AzureADUserListWorksheet.Cells.Item(1,13) = 'Country' 
    $AzureADUserListWorksheet.Cells.Item(1,14) = 'CompanyName' 
    $AzureADUserListWorksheet.Cells.Item(1,15) = 'Manager' 
    $AzureADUserListWorksheet.Cells.Item(1,16) = 'TelephoneNumber' 
    $AzureADUserListWorksheet.Cells.Item(1,17) = 'Mobile' 
    $AzureADUserListWorksheet.Cells.Item(1,18) = 'AccountEnabled' 
    $AzureADUserListWorksheet.Cells.Item(1,19) = 'UserType'  
    $AzureADUserListWorksheet.Cells.Item(1,20) = 'ObjectType'          
    $AzureADUserListWorksheet.Cells.Item(1,21) = 'AD_Id'           
    # Cell Counter 
    $row_counter = 3 
    $column_counter = 1 
  } 
  
  # Iterating over the AZURE users  
  foreach ($users_iterator in $users)
  { 
    $user_object_id = $users_iterator.objectid           
    $user_UserPrincipalName = $users_iterator.UserPrincipalName
    $user_Display_Name = $users_iterator.DisplayName
    $user_GivenName = $users_iterator.GivenName
    $user_Surname = $users_iterator.Surname
    $user_Department = $users_iterator.Department
    $user_JobTitle = $users_iterator.JobTitle
    $user_EMail = $users_iterator.Mail
    $user_StreetAddress = $users_iterator.StreetAddress
    $user_City = $users_iterator.City
    $user_State = $users_iterator.State
    $user_PostalCode = $users_iterator.PostalCode
    $user_Country = $users_iterator.Country
    $user_CompanyName = $users_iterator.CompanyName
    $user_TelephoneNumber = $users_iterator.TelephoneNumber
    $user_Mobile = $users_iterator.Mobile
    $user_AccountEnabled = $users_iterator.AccountEnabled
    $user_UserType = $users_iterator.UserType
    $user_ObjectType = $users_iterator.ObjectType

    if ($user_object_id -ne $null)
    { 
      #$user_Manager = (Get-AzureADUserManager -ObjectID $user_object_id).DisplayName

      $user_ManagerID = (Get-AzureADUserManager -ObjectID $user_object_id).ObjectID
      if ($user_ManagerID -eq $null) 
      { 
        $user_Manager = "NULL"
      } else { 
        $user_Manager = (Get-AzureADUserExtension -ObjectID $user_ManagerID).onPremisesDistinguishedName 
        if($user_Manager -eq $null) 
        { 
          $user_Manager =$user_ManagerID
        }
      }

      $user_DNObjectID = (Get-AzureADUserExtension -ObjectID $user_object_id).onPremisesDistinguishedName
      if($user_DNObjectID -eq $null)
      { 
        $user_DNObjectID =$user_object_id
      }
    } else { 
      $user_Manager = "NULL"
      $user_DNObjectID = "NULL"
    } 
 
    Write-host "Extracting information for user: " $user_Display_Name 

    #Build Excel ROW
    if ($ExcelFileName -ne "") 
    {

      if ($FilterPhoneRequired) 
      {
        if ($user_TelephoneNumber.length -lt 1)  
        { 
          $Add = $false 
        } else {
          $Add=$true
        }
      } else {
        $Add=$true
      }
      if ($Add)
      {
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_DNObjectID
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_UserPrincipalName
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Display_Name 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_GivenName 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Surname 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Department 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_JobTitle 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_EMail 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_StreetAddress
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_City 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_State 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_PostalCode 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Country 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_CompanyName 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Manager 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_TelephoneNumber 
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_Mobile
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_AccountEnabled
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_UserType
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_ObjectType
        $AzureADUserListWorksheet.Cells.Item($row_counter,$column_counter++) = $user_object_id
        $row_counter = $row_counter + 1 
      }
    }

    $column_counter = 1 
 
    #Mark item as found
    $i=0
    $FoundItem=-1
    $RemovedItem=-1
    foreach ($Currenlist_user in $CurrentList)
    {
      if ($Currenlist_user["Title"] -eq $user_object_id)  
      { 
        if ($FilterPhoneRequired)
        {
          if ($Currenlist_user["TelephoneNumber"].length -gt 1)
          {
            $FoundList[$i]=1
		        $FoundItem=$i
          } else {
            Write-Host "Entry to be removed - no phone nr" -ForegroundColor Green 
            $FoundList[$i]=0
            $RemovedItem=$i
          }
        } else {
          $FoundList[$i]=1
		      $FoundItem=$i
        }  
      }
      $i++
    }
    
    #Check if update is necessary  
    if ($FoundItem -ne -1)
    {
      $ProceedUpdate=$false
      if ($CurrentList[$FoundItem]["Display_Name"] -ne $user_Display_Name) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["Surname"] -ne $user_Surname) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["GivenName"] -ne $user_GivenName) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["CompanyName"] -ne $user_CompanyName) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["JobTitle"] -ne $user_JobTitle) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["Department"] -ne $user_Department) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["EMail"] -ne $user_EMail) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["TelephoneNumber"] -ne $user_TelephoneNumber) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["Mobile"] -ne $user_Mobile) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["StreetAddress"] -ne $user_StreetAddress) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["City"] -ne $user_City) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["State"] -ne $user_State) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["PostalCode"] -ne $user_PostalCode) {
        $ProceedUpdate=$true
      }
	    if ($CurrentList[$FoundItem]["Country"] -ne $user_Country) {
        $ProceedUpdate=$true
      }
      if ($CurrentList[$FoundItem]["ObjectID"] -ne $user_DNObjectID) {
        $ProceedUpdate=$true
      }
      if ($CurrentList[$FoundItem]["UserPrincipalName"] -ne $user_UserPrincipalName) {
        $ProceedUpdate=$true
      }
      if ($CurrentList[$FoundItem]["Manager"] -ne $user_Manager) {
        $ProceedUpdate=$true
      }

      if ($ProceedUpdate) 
      {
        #If any change detected, go ahead with update
        #Get List Item to Update by Query (Query the SharePoint list for the current item iteration)
        $ListItemTitle=$user_object_id
        $ListItem = Get-PnPListItem -List $ListName -Query "<View><Query><Where><Eq><FieldRef Name='Title'/><Value Type='Text'>$ListItemTitle</Value></Eq></Where></Query></View>" -PageSize 1 -ErrorAction Stop
        
        if($ListItem -ne $null)
        { 
          #Update SharePoint List Item 
          $file = Set-PnPListItem -List $ListName -Identity $ListItem -Values @{"Title" = $user_object_id; "Display_Name"=$user_Display_Name;"Surname"=$user_Surname;"GivenName"=$user_GivenName;"CompanyName"=$user_CompanyName;"JobTitle"=$user_JobTitle;"Department"=$user_Department;"EMail"=$user_EMail ;"TelephoneNumber"=$user_TelephoneNumber;"Mobile"=$user_Mobile;"StreetAddress"=$user_StreetAddress;"City"=$user_City;"State"=$user_State ;"PostalCode"=$user_PostalCode ; "Country"=$user_Country;"ObjectID"=$user_DNObjectID; "UserPrincipalName"=$user_UserPrincipalName;"Manager"= $user_Manager}
          $ErrorMsg="Update Entry "+$user_object_id
          Write-Host $ErrorMsg -ForegroundColor Green 
        } else {
          ##Issue could not update
          $ErrorMsg="Record could not be updated: "+$user_object_id+". `n"
          Write-Host $ErrorMsg -ForegroundColor Yellow 
        }
      }
    } else {  
      if ($FilterPhoneRequired) {
        if ($user_TelephoneNumber.length -lt 1)  { 
          $Add = $false 
        } else {
          $Add=$true
        }
      } else {
        $Add=$true
      }
      if ($Add)
      {
        #Add SharePoint List item
        $file = Add-PnPListItem -List $ListName -Values @{"Title" = $user_object_id; "Display_Name"=$user_Display_Name;"Surname"=$user_Surname;"GivenName"=$user_GivenName;"CompanyName"=$user_CompanyName;"JobTitle"=$user_JobTitle;"Department"=$user_Department;"EMail"=$user_EMail ;"TelephoneNumber"=$user_TelephoneNumber;"Mobile"=$user_Mobile;"StreetAddress"=$user_StreetAddress;"City"=$user_City;"State"=$user_State ;"PostalCode"=$user_PostalCode ; "Country"=$user_Country;"ObjectID"=$user_DNObjectID; "UserPrincipalName"=$user_UserPrincipalName;"Manager"= $user_Manager}
      } else {
        if($RemovedItem -ne -1){
          # This item was deleted
        } else {
          # This item is skipped because it does not meet the requirement
          # Write-Host "Entry not included - does not meet filtering requirement" -ForegroundColor Green 
        }
      }
    }
  
    #End of Azure item iteration
  } 
 
  #Removing from SP List the users we haven't found in Azure
  if ($RemoveNotFoundADUsersFromSP)
  {
    #Remove not found items 
    $i=0
    foreach ($FoundList_user in $FoundList)
    {
      if ($CurrentList[$i]["Title"].StartsWith("KEEP ")) 
      {
        #Skip this entry
      } else {
        if ($FoundList_user -eq 0) 
        {
          Write-host "Remove not found item from SP List: " $CurrentList[$i]["Title"]
          Remove-PnPListItem -List $ListName -Id $CurrentList[$i]["ID"] -Force 
        }
      }
      $i++
    }
  }

  #End of function     
} 
 
## Calling function 
CreateAzureUserList $ExcelFileName $ExcelFolderPath $FilterPhoneRequired
 
if ($ExcelFileName -ne "") 
{ 
  $ExcelFilePath = join-path -path $ExcelFolderPath -childpath $ExcelFileName
  # Checking if the xlsx already exists 
  if(Test-Path $ExcelFilePath)
  { 
    Write-Host "File already exists. Deleting the current file and creating a new one. `n" -ForegroundColor Yellow 
    Remove-Item $ExcelFilePath 
    # Saving the workbook/excel file 
    $workbook.SaveAs($ExcelFilePath) 
  } else { 
    # Saving the workbook/excel file 
    $workbook.SaveAs($ExcelFilePath) 
  } 
  
  Write-Host "Saving file. `n" -ForegroundColor Green 
  $excel.Quit() 
}